@@ -1,4 +1,4 @@
-__version__ = "1.0.0"
+__version__ = "1.0.1"
 
 def quant_test_manual():
     import torch.nn as nn
@@ -107,7 +107,6 @@ def quant_test_auto():
     import torch.nn as nn
     import torch.nn.functional as F
 
-    import micronet.compression.quantization.wbwtab.quantize as quant_wbwtab
     import micronet.compression.quantization.wqaq.dorefa.quantize as quant_dorefa
     import micronet.compression.quantization.wqaq.iao.quantize as quant_iao
 
@@ -131,15 +130,12 @@ def quant_test_auto():
             return F.log_softmax(x, dim=1)
 
     lenet = LeNet()
-    quant_lenet_wbwtab = quant_wbwtab.prepare(lenet, inplace=False)
     quant_lenet_dorefa = quant_dorefa.prepare(lenet, inplace=False)
     quant_lenet_iao = quant_iao.prepare(lenet, inplace=False)
 
     print('***ori_model***\n', lenet)
-    print('\n***quant_model_wbwtab***\n', quant_lenet_wbwtab)
     print('\n***quant_model_dorefa***\n', quant_lenet_dorefa)
     print('\n***quant_model_iao***\n', quant_lenet_iao)
 
     print('\nquant_model is ready')
     print('micronet is ready')
-    
\ No newline at end of file
